from flask import render_template, abort, redirect, url_for, abort, flash, request, current_app, make_response, g
from flask_login import login_required, login_user, logout_user
from flask_sqlalchemy import get_debug_queries

from . import admin
from .forms import ChargeForm

from .. import db
from ..email import send_email
from ..models import User, Transaction, Deployment, Service, Region, Address, Domain, contact_proxmaster
from ..decorators import admin_required, permission_required

import base64
import string
import random
from datetime import datetime, timedelta, date, time
import ast

#@admin.before_app_request
#def before_request():
#    g.user = current_user
#    print('current_user: %s, g.user: %s, leaving bef_req' % (current_user, g.user))

@admin.after_app_request
def after_request(response):
    for query in get_debug_queries():
        if query.duration >= current_app.config['SLOW_DB_QUERY_TIME']:
            current_app.logger.warning('Slow query: %s\nParameters: %s\nDuration: %fs\nContext: %s\n' % (query.statement, query.parameters, query.duration, query.context))
    return response

@admin.route("/", methods=['GET'])
@login_required
@admin_required
def index():
    return redirect(url_for('admin.list_users'))

@admin.route("/listitems", methods=['GET'])
@login_required
@admin_required
def list_items():
    alldeployments = Deployment.query.order_by(Deployment.date_last_charge.asc()).all()
    alldomains = Domain.query.order_by(Domain.date_expire.asc()).all()
    allservices = Service.query.order_by(Service.date_last_charge.asc()).all()
    alladdresses = Address.query.order_by(Address.user_id.asc()).all()
    return render_template('admin/list_items.html', deployments=alldeployments, domains=alldomains, services=allservices, addresses=alladdresses)

@admin.route("/listusers", methods=['GET'])
@login_required
@admin_required
def list_users():
    allusers = User.query.order_by(User.last_seen.desc()).all()
    return render_template('admin/list_users.html', users=allusers)

@admin.route("/charge/<int:user_pid>", methods=['GET', 'POST'])
@login_required
@admin_required
def charge(user_pid=0):
    cuser = User.query.filter_by(pid=user_pid).first()
    form = ChargeForm()
    if form.validate_on_submit():
        transaction = Transaction(user_id=int(cuser.pid), description='Account charged by staff', value=float(form.amount.data))
        db.session.add(transaction)
        db.session.commit()

        cuser.wallet += float(form.amount.data)
        db.session.add(cuser)
        db.session.commit()
        return redirect(url_for('admin.list_users'))
    return render_template('admin/charge.html', form=form, usr=cuser)



@admin.route("/dashboard/<int:user_pid>", methods=['GET'])
@login_required
@admin_required
def dashboard(user_pid=0):
    cuser = User.query.filter_by(pid=user_pid).first()
    inv_deployments = cuser.inv_deployments.filter_by(enabled=True).order_by(Deployment.date_created.desc())
    inv_deploycubeids = []
    inv_deployments_list = []
    for invcls in inv_deployments:
        inv_deploycubeids.extend([invcls.machine_id])
        inv_deployments_list.extend([invcls.machine_alias])

    inv_services = cuser.inv_services.filter_by(enabled=True).order_by(Service.date_last_charge.asc())
    inv_services_list = []
    for invcls in inv_services:
        inv_services_list.extend([invcls.description])

    inv_domains = cuser.inv_domains.filter_by(enabled=True).order_by(Domain.date_created.desc())
    inv_domains_list = []
    for invcls in inv_domains:
        if invcls.user_id == cuser.pid and invcls.enabled == True:
            inv_domains_list.extend([invcls.fqdn])

    inv_addresses = cuser.inv_addresses.filter_by(enabled=True).order_by(Address.ip.asc())
    inv_addresses_list = []
    for invcls in inv_addresses:
        if invcls.user_id == cuser.pid and invcls.enabled == True:
            inv_addresses_list.extend([invcls.ip])

    #extract rrd and status from the deployments
    rrd = {}
    statuses = {}
    for cubeid in inv_deploycubeids:
        rrd[cubeid] = {}
        try:
            query = contact_proxmaster({}, 'vmrrd', cubeid)
        except:
            flash('Deploy #{} unreachable.'.format(str(cubeid)))

        graphs_list = ['net', 'cpu', 'mem', 'hdd']
        try: 
            for graph in graphs_list:
                raw = query[graph]['image'].encode('raw_unicode_escape')
                rrd[cubeid][graph] = base64.b64encode(raw).decode()
            status = { cubeid : query['status'] }
            statuses.update(status)
        except Exception as e:
            print(e)
            flash('Deploy #{} unreachable. Support is notified'.format(str(cubeid)))
            send_email(current_app.config['MAIL_USERNAME'], 'Cube {} is unreachable'.format(cubeid), 
                                          'vmanager/email/adm_unreachable', user=cuser, cubeid=cubeid )
    current_app.logger.info('[ADMIN] {} deployments: {}, services: {}, domains: {}, services: {}'.format(cuser.email, inv_deployments_list, inv_services_list, inv_domains_list, inv_addresses_list ))
    return render_template('vmanager/dashboard.html', rrd=rrd, status=statuses, inv_deployments=inv_deployments, inv_services=inv_services, inv_domains=inv_domains, inv_addresses=inv_addresses)

@admin.route("/listtransactions", methods=['GET'])
@login_required
@admin_required
def list_transactions():
    alltransactions = Transaction.query.order_by(Transaction.date_created.desc()).all()
    return render_template('admin/list_transactions.html', transactions=alltransactions)

@admin.route("/transaction/<int:user_pid>", methods=['GET'])
@login_required
@admin_required
def transaction(user_pid=0):
    cuser = User.query.filter_by(pid=user_pid).first()
    transactions = cuser.inv_transactions.order_by(Transaction.date_created.desc()).all()

    labelslist = ['today']
    translist = [cuser.wallet]
    prevvalue = cuser.wallet
    for tr in transactions:
        labelslist.insert(0, str(tr.date_created.strftime('%d.%m')))
        translist.insert(0, prevvalue - tr.value)
        prevvalue -= tr.value

    if len(labelslist) <= 1:
        labelslist.insert(0, 'before')
        translist.insert(0, 0)

    #current_app.logger.info('[{}] transactions: {} {} '.format(cuser.email, translist, labelslist))
    return render_template('uinvoice/transactions.html', transactions=transactions, translist=translist, labelslist=labelslist, cuser=cuser)

